/* 
TN: Tutorial Notes
- These are 2 Controllers for the simple demo-app 
- Requires the feedbackService as a dependency, as it is the correct way to architect an AngularJS app
    by placing the service outside of the controller

Very important note on BEST-PRACTICES
I found loads of samples which work, but were so trivial you shouldn't imitate the coding style
...unfortunately, I know that everybody just copies what they find, because it works
...resulting it terrible code, hard to maintain. 
Because of this, we chose to structure this differently, as far as possible alligned with
the AngularJS Styleguide by John Papa https://github.com/johnpapa/angularjs-styleguide


*/
(function () { // TN: this is a helper construct, research iife or read https://github.com/johnpapa/angularjs-styleguide#iife

    // Training Notes: the following three lines create a new app (= module) and define 3 controllers
    //      The controllers are initialized through functions which follow later on
    //      Note: the syntax is a bit different than many examples. We're following 
    //      ...best practices as suggested by John Papa https://github.com/johnpapa/angularjs-styleguide#named-vs-anonymous-functions 
    angular.module("FeedbackApp", ['2sxc4ng', 'FeedbackServices'])
        .constant('createdBy', '2sic')          // just a demo how to use constant or value configs in AngularJS
        .constant('licence', 'MIT')             // these wouldn't be necessary, just added for learning exprience
        .controller("New", NewController)
        .controller("Admin", AdminController)
        .controller("EditDialog", EditDialogController);




    // The form controller which creates/initializes the ViewModel and all actions involved
    // --------------------------------------------------
    // Basically handles the "new" dialog
    // The only specials are
    // - it first creates an empty item with all properties to allow easy binding. later on the item (vm.item) can be sent without any conversion to the server for saving
    // - the save function will do some show/hide dialogs which is quite nice
    // - the binding of the categories does not happen here, instead it happens in the sub-controller EditDialog
    function NewController(categoriesSvc, feedbackSvc) {
        // Training Notes: use the vm = this as recommended by https://github.com/johnpapa/angularjs-styleguide#controlleras-with-vm
        var vm = this;
        vm.show = "form";
        vm.item = feedbackSvc.new(); 

        // ensure the categories are loaded and select the first one
         categoriesSvc.load().then(function (result) {
             vm.item.CategoryId = result.data[0].Id;
         });

        // the send-method, will first show "sending", then a thank-you message
        vm.save = function () {
            vm.show = 'sending';
            feedbackSvc.add(vm.item).then(function () {
                vm.show = 'thanks'
            })
        }
    };








    // Constructs the admin-controller
    function AdminController($sce, sxc, feedbackSvc) { 
        var vm = this;
        vm.show = "list";
        vm.item = null;

        // this list is bound to the live-list of feedbacks. so if the service updates the feedbacks later, it will auto-bind again
        vm.feedbacks = feedbackSvc.allLive();

        // Some basic functions, which are just the same function on the service
        vm.del = feedbackSvc.del;
        vm.setDone = feedbackSvc.setDone;
        vm.setNew = feedbackSvc.setNew;

        // change UI to review dialog
        vm.review = function (feedbackId) {
            feedbackSvc.get(feedbackId).then(function (response) {
                vm.item = response.data.Feedback[0];
                vm.show = 'form';
            });
        }

        // change UI back to list dialog
        vm.backToList = function backToList() {
            vm.item = null;
            vm.show = 'list';
        }

        // save changes
        vm.save = function () {
            feedbackSvc.update(vm.item);
            vm.show = 'list';
        }

        // will set the status to the other-status depending on the current one
        vm.swapStatus = function (feedback) {
            var action = (feedback.Status == "New") // choose which action to execute
                ? feedbackSvc.setDone
                : feedbackSvc.setNew;
            action(feedback.Id);    // execute the action, then refresh
        }

        // helper function to provide a 2sxc-toolbar buttons when in edit-mode
        vm.toolbar = function (feedback) {
            return sxc.manage ? $sce.trustAsHtml(sxc.manage.getToolbar({ entityId: feedback.Id })) : "";
        }

    }


    // The Edit-Controller is very trivial
    // It only includes extra functionality in the edit dialog like the list of categories needed
    // Not that it doesn't have any save-actions or similar, as that data resides in the main view/controller
    // It would also be easy to keep this in the main controller, but it's nice to separate things as much as possible
    function EditDialogController(categoriesSvc) {
        var vm = this;
        vm.categories = categoriesSvc.allLive();
    }




} ());