/* Introduction for all those new to AngularJS

This is a service-factory. It is basically a component which is responsible for
providing data to any other Angular-subsystem and/or help the other systems
get something done (in this case, send data back to the server). 

Note: it's implemented as a factory, so that it can remember the ID of the module by itself
if it were a service instead of a factory, it would be a singleton (reusable, but always the same object)
That would cause problems if the same app was included multiple times on a page

*/ 

// Note that I'm demonstrating multiple factories (categories & feedback)
// I always recommend the minification-safe syntax as 
angular.module('FeedbackServices', ['ng', '2sxc4ng'])

    // The categoriesSvc is the service for managing categories in this mini-app
    // Notice that it does very nothing, but keeping it separate is just good code-style
    // Of course without all these comments it's just a few lines, and very elegant to maintain. 
    // It shows a technique, where the list of something is cached on the service
    // So categories is created empty, and you can already bind to it.
    // Later on it will be filled (not replaced), so the binding will automatically show the late-filling
    // Without the caching-feature it would be a 5-line-service, but the data binding in the controller would need more code
    // Not that this also contains a demo for promise-chaining, as the result is first processed, then returned as a promise
    .factory('categoriesSvc', ['$http', function($http) { 
        var svc = {};          

        // The all starts as an empty list. It will later be populated, but you can already bind to it
        svc._all = [];

        // retrieve a bindable list of all, even if loaded later
        svc.allLive = function allLive() {
            if(svc._all.length == 0)
                svc.load();
            return svc._all;
        }

        // load and return a promise, but also update internal cache whenever data is retrieved
        svc.load = function load() {
            return $http.get("app-api/Feedback/Categories")
                .then(svc.updateLiveAll);
        }

        // auto-load when the service is started
        // probably not a good idea...svc.load();

        // use a promise-result to re-fill the live list of all items, return the promise again
        svc.updateLiveAll = function updateLiveAll(result) {
            svc._all.length = 0; // clear
            for(i = 0; i < result.data.length; i++)
                svc._all.push(result.data[i]);
            svc._all[0].isDefault = true;
            return result;
        }

        return svc;
    }])

    // The feedbackSvc is in charge of all feedback thnigs
    // It has both reading-abilities (get one or many), various state-update methods and add/update/delete
    .factory('feedbackSvc', ['$http', function feedbackSvc($http) {
        var svc = {};               

        // Cached list of all feedbacks
        svc._all = [];
        svc.allLive = function getAllLive(){
            if(svc._all.length == 0)
                svc.get();
            return svc._all;
        }



        // This request shows how to use $http within the dnn/2sxc context. 
        // the relative url starting with app-api or similar causes it to be auto-rewritten before use
        // I'm providing the get one/all in the same function, if ID is supplie, get that, else all
        svc.get = function get(id) {
            return id 
                ? $http.get("app-api/Feedback/Get", { params: { "id": id } })
                : $http.get("app-api/Feedback/All").then(svc.updateLiveAll);
        }

        // use a promise-result to re-fill the live list of all items, return the promise again
        svc.updateLiveAll = function updateLiveAll(result) {
            svc._all.length = 0; // clear
            for(i = 0; i < result.data.length; i++)
                svc._all.push(result.data[i]);
            return result;
        }

        // just refresh the all
        svc.refreshAll = function refreshAll(promise){
            svc.get();
            return promise;
        }


        svc.add = function add(item) {
            return $http.post("app-api/Feedback/Create", item);
        }

        svc.update = function update(feedback){
            return $http.post('app-api/Feedback/Update', feedback, { params: { 'id': feedback.Id }})
                .then(svc.refreshAll);
        }

        // This delivers a new, empty feedback item
        svc.new = function() {
            return {
                Id: 0,
                Subject: "",
                Message: "",
                CategoryId: null
            }
        }

        svc.setDone = function(id) {
            return $http.get("app-api/Feedback/SetComplete", { params: { 'id': id }})
                .then(svc.refreshAll);
        }

        svc.setNew = function(id) {
            return $http.get("app-api/Feedback/SetNew", { params: { 'id': id }})
                .then(svc.refreshAll);
        }

        svc.del = function(id) {
            return $http.delete("app-api/Feedback/Delete", { params: { 'id': id }})
                .then(svc.refreshAll);
        }
            
        return svc;
    }]);